import { Hono } from "hono";
import { youtubePlayerParsing } from "../../lib/helpers/youtubePlayerHandling.ts";
import {
    reasonBot,
    subreasonProtectCommunity,
    videoRestricted,
    videoUnavailable,
} from "metrics";

const player = new Hono();

const errors = [
    {
        check: (yt: object) =>
            // @ts-ignore: Property 'playabilityStatus' does not exist on type 'object'
            yt.playabilityStatus?.reason?.includes(
                "Sign in to confirm you’re not a bot",
            ),
        action: () => reasonBot.inc(),
    },
    {
        check: (yt: object) =>
            // @ts-ignore: Property 'playabilityStatus' does not exist on type 'object'
            yt.playabilityStatus?.errorScreen?.playerErrorMessageRenderer
                ?.subreason?.runs?.[0]?.text?.includes(
                    "This helps protect our community",
                ),
        action: () => subreasonProtectCommunity.inc(),
    },
    {
        check: (yt: object) =>
            // @ts-ignore: Property 'playabilityStatus' does not exist on type 'object'
            yt.playabilityStatus?.errorScreen?.playerErrorMessageRenderer
                ?.subreason?.runs?.[0]?.text?.includes("Video unavailable"),
        action: () => videoUnavailable.inc(),
    },
    {
        check: (yt: object) =>
            // @ts-ignore: Property 'playabilityStatus' does not exist on type 'object'
            yt.playabilityStatus?.reason?.includes("This video is restricted"),
        action: () => videoRestricted.inc(),
    },
];

player.post("/player", async (c) => {
    const jsonReq = await c.req.json();
    const innertubeClient = c.get("innertubeClient");
    const konfigStore = c.get("konfigStore");
    if (jsonReq.videoId) {
        const yt = await youtubePlayerParsing({
            innertubeClient,
            videoId: jsonReq.videoId,
            konfigStore,
            tokenMinter: c.get("tokenMinter"),
        });
        errors.forEach((error) => {
            if (error.check(yt)) {
                error.action();
            }
        });
        return c.json(yt);
    }
});

export default player;
